@@ -109,8 +109,10 @@ def __checkLinkbotTTY(comport):
 def _unpack(fmt, buffer):
   if sys.version_info[0] == 2 and sys.version_info[1] == 6:
     return struct.unpack(fmt, bytes(buffer))
-  else:
+  elif sys.version_info[0] == 3:
     return struct.unpack(fmt, buffer)
+  else:
+    return struct.unpack(fmt, str(buffer))
 
 
 class BaroboException(Exception):
@@ -133,6 +135,8 @@ class BaroboCtx():
   EVENT_REPORTADDRESS = 0x21
   TWI_REGACCESS = 0x22
   EVENT_DEBUG_MSG = 0x23
+  EVENT_JOINT_MOVED = 0x24
+  EVENT_ACCEL_CHANGED = 0x25
 
 
   CMD_STATUS = 0x30
@@ -221,6 +225,10 @@ class BaroboCtx():
   CMD_PING = 0x89
   CMD_GET_HW_REV = 0x8A
   CMD_SET_HW_REV = 0x8B
+  CMD_SET_JOINT_EVENT_THRESHOLD = 0x8C
+  CMD_SET_ENABLE_JOINT_EVENT = 0x8D
+  CMD_SET_ACCEL_EVENT_THRESHOLD = 0x8E
+  CMD_SET_ENABLE_ACCEL_EVENT = 0x8F
 
   MOTOR_FORWARD = 1
   MOTOR_BACKWARD = 2
@@ -329,7 +337,9 @@ class BaroboCtx():
       self.__checkStatus()
       self.__getDongleID()
     except:
-      raise BaroboException('Could not connect to dongle at {0}'.format(ttyfilename))
+      self.phys.close()
+      self.link.stop()
+      self.connectDongleSFP(ttyfilename)
 
   def connectDongleSFP(self, ttyfilename):
     """
@@ -346,7 +356,9 @@ class BaroboCtx():
       raise BaroboException('Could not connect to dongle at {0}'.format(ttyfilename))
 
   def disconnect(self):
+    self.link.stop()
     self.phys.disconnect()
+    self.children = []
 
   def handlePacket(self, packet):
     self.readQueue.put(packet)
