@@ -13,11 +13,16 @@ import barobo
 
 import ctypes
 import ctypes.util
+import os
 
 try:
   from sys import platform as platform
   if "win32" == platform:
-    _sfp = ctypes.CDLL("libsfp.dll")
+    for p in sys.path:
+      fname = os.path.join(p, "libsfp.dll")
+      if os.path.isfile(fname):
+        _sfp = ctypes.CDLL("libsfp.dll")
+        break
   else:
     _sfp = ctypes.CDLL(ctypes.util.find_library("sfp"))
   haveSFP = True
@@ -48,6 +53,7 @@ if havePySerial:
       self.timeout = None
 
     def disconnect(self):
+      self.close()
       pass
 
 class PhysicalLayer_Socket(socket.socket):
@@ -130,12 +136,16 @@ class LinkLayer_Base:
     self.phys = physicalLayer
     self.deliver = readCallback
     self.writeLock = threading.Lock()
+    self.stopflag = False
 
   def start(self):
     self.thread = threading.Thread(target=self._run)
     self.thread.daemon = True
     self.thread.start()
 
+  def stop(self):
+    self.stopflag = True
+
 class LinkLayer_TTY(LinkLayer_Base):
   def __init__(self, physicalLayer, readCallback):
     LinkLayer_Base.__init__(self, physicalLayer, readCallback)
@@ -161,6 +171,8 @@ class LinkLayer_TTY(LinkLayer_Base):
     self.phys.flushOutput()
     while True:
       byte = self.phys.read()
+      if self.stopflag:
+        break
       if byte is None:
         continue
       if DEBUG:
@@ -256,6 +268,8 @@ class LinkLayer_SFP(LinkLayer_Base):
     _sfp.sfpConnect(self.ctx)
     while True:
       byte = self.phys.read()
+      if self.stopflag:
+        break
       if byte is None:
         continue
       if DEBUG:
@@ -266,6 +280,8 @@ class LinkLayer_SFP(LinkLayer_Base):
       else:
         octet = byte[0]
       rc = _sfp.sfpDeliverOctet(self.ctx, octet, None, 0, None)
+    if DEBUG:
+      print ("SFP Link Layer stopping...")
 
 class LinkLayer_Socket(LinkLayer_Base):
   def __init__(self, physicalLayer, readCallback):
@@ -286,6 +302,8 @@ class LinkLayer_Socket(LinkLayer_Base):
     self.phys.flushOutput()
     while True:
       byte = self.phys.read()
+      if self.stopflag:
+        break
       if DEBUG:
         print ("Byte: {0}".format(list(map(hex, bytearray(byte)))))
       if byte is None:
